'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var web = require('solid-js/web');
var solidJs = require('solid-js');

/* Original Code written by Oliver Geer (WebCoder49) */
// https://github.com/WebCoder49/code-input/blob/main/plugins/indent.js

function handleTabKey(event, input_element, code) {
  event.preventDefault(); // stop normal

  if (!event.shiftKey && input_element.selectionStart == input_element.selectionEnd) {
    // Shift always means dedent - this places a tab here.
    let before_selection = code.slice(0, input_element.selectionStart); // text before tab
    let after_selection = code.slice(input_element.selectionEnd, input_element.value.length); // text after tab

    let cursor_pos = input_element.selectionEnd + 1; // where cursor moves after tab - moving forward by 1 char to after tab
    input_element.value = before_selection + '\t' + after_selection; // add tab char

    // move cursor
    input_element.selectionStart = cursor_pos;
    input_element.selectionEnd = cursor_pos;
  } else {
    let lines = input_element.value.split('\n');
    let letter_i = 0;
    let selection_start = input_element.selectionStart; // where cursor moves after tab - moving forward by 1 indent
    let selection_end = input_element.selectionEnd; // where cursor moves after tab - moving forward by 1 indent

    let number_indents = 0;
    let first_line_indents = 0;
    for (let i = 0; i < lines.length; i++) {
      letter_i += lines[i].length + 1; // newline counted
      if (input_element.selectionStart <= letter_i && input_element.selectionEnd >= letter_i - lines[i].length) {
        // Starts before or at last char and ends after or at first char
        if (event.shiftKey) {
          if (lines[i][0] == '\t') {
            // Remove first tab
            lines[i] = lines[i].slice(1);
            if (number_indents == 0) first_line_indents--;
            number_indents--;
          }
        } else {
          lines[i] = '\t' + lines[i];
          if (number_indents == 0) first_line_indents++;
          number_indents++;
        }
      }
    }
    input_element.value = lines.join('\n');

    // move cursor
    input_element.selectionStart = selection_start + first_line_indents;
    input_element.selectionEnd = selection_end + number_indents;
  }
}
function handleEnterKey(event, input_element) {
  event.preventDefault();
  let lines = input_element.value.split('\n');
  let letter_i = 0;
  let current_line = lines.length - 1;
  let new_line = '';
  let number_indents = 0;

  // find the index of the line our cursor is currently on
  for (let i = 0; i < lines.length; i++) {
    letter_i += lines[i].length + 1; // newline counted
    if (input_element.selectionStart <= letter_i) {
      current_line = i;
      break;
    }
  }

  // count the number of indents the current line starts with (up to our cursor position in the line)
  let cursor_pos_in_line = lines[current_line].length - (letter_i - input_element.selectionEnd) + 1;
  for (let i = 0; i < cursor_pos_in_line; i++) {
    if (lines[current_line][i] == '\t') {
      number_indents++;
    } else {
      break;
    }
  }

  // determine the text before and after the cursor and chop the current line at the new line break
  let text_after_cursor = '';
  if (cursor_pos_in_line != lines[current_line].length) {
    text_after_cursor = lines[current_line].substring(cursor_pos_in_line);
    lines[current_line] = lines[current_line].substring(0, cursor_pos_in_line);
  }

  // insert our indents and any text from the previous line that might have been after the line break
  for (let i = 0; i < number_indents; i++) {
    new_line += '\t';
  }
  new_line += text_after_cursor;

  // save the current cursor position
  let selection_start = input_element.selectionStart;
  let selection_end = input_element.selectionEnd;

  // splice our new line into the list of existing lines and join them all back up
  lines.splice(current_line + 1, 0, new_line);
  input_element.value = lines.join('\n');

  // move cursor to new position
  input_element.selectionStart = selection_start + number_indents + 1; // count the indent level and the newline character
  input_element.selectionEnd = selection_end + number_indents + 1;
}

function styleInject(css, ref) {
  if (ref === void 0) ref = {};
  var insertAt = ref.insertAt;
  if (!css || typeof document === 'undefined') {
    return;
  }
  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';
  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }
  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = "/* Original CSS written by Oliver Geer (WebCoder49) */\n/* https://css-tricks.com/creating-an-editable-textarea-that-supports-syntax-highlighted-code/ */\n/* Updated for solid-code-input */\n\n.styles-module_wrap__VKL-2 {\n  position: relative;\n  display: block;\n  min-height: 60px;\n  box-sizing: border-box;\n  position: relative;\n  top: 0;\n  left: 0;\n  display: block;\n  overflow: hidden;\n  min-height: 76px;\n  font-size: 16px;\n  font-family: monospace;\n  line-height: normal;\n  tab-size: 2;\n  caret-color: darkgrey;\n  white-space: pre;\n}\n\n.styles-module_wrap__VKL-2 textarea,\n.styles-module_wrap__VKL-2 pre {\n  width: 100%;\n  height: 100%;\n  border: none;\n  overflow: auto;\n  outline: none;\n  position: absolute;\n  top: 0;\n  left: 0;\n  font-family: monospace;\n  font-size: inherit !important;\n  line-height: inherit !important;\n  background: transparent;\n  padding: 0 !important;\n  margin: 0 !important;\n}\n\n.styles-module_wrap__VKL-2 textarea,\n.styles-module_wrap__VKL-2 pre,\n.styles-module_wrap__VKL-2 pre * {\n  font-size: inherit !important;\n  font-family: inherit !important;\n  line-height: inherit !important;\n  tab-size: inherit !important;\n}\n\n.styles-module_wrap__VKL-2 textarea {\n  display: block;\n  color: transparent;\n  background: transparent;\n  caret-color: inherit !important;\n  z-index: 1;\n}\n\n.styles-module_wrap__VKL-2 pre::-webkit-scrollbar {\n  width: 0;\n  height: 0;\n}\n\n.styles-module_wrap__VKL-2 pre {\n  z-index: 0;\n}\n\n.styles-module_wrap__VKL-2 textarea,\n.styles-module_wrap__VKL-2 pre {\n  overflow: auto !important;\n  white-space: inherit;\n  word-spacing: normal;\n  word-break: normal;\n  word-wrap: normal;\n}\n\n.styles-module_wrap__VKL-2 textarea.styles-module_resize-both__x-us- {\n  resize: both;\n}\n\n.styles-module_wrap__VKL-2 textarea.styles-module_resize-none__0Stjz {\n  resize: none;\n}\n\n.styles-module_wrap__VKL-2 textarea.styles-module_resize-horizontal__Gt6Mt {\n  resize: horizontal;\n}\n\n.styles-module_wrap__VKL-2 textarea.styles-module_resize-vertical__eZhdp {\n  resize: vertical;\n}\n";
var styles = {"wrap":"styles-module_wrap__VKL-2","resize-both":"styles-module_resize-both__x-us-","resize-none":"styles-module_resize-none__0Stjz","resize-horizontal":"styles-module_resize-horizontal__Gt6Mt","resize-vertical":"styles-module_resize-vertical__eZhdp"};
styleInject(css_248z);

const _tmpl$ = /*#__PURE__*/web.template(`<div style="padding: 1rem; background: red;"><div><textarea></textarea><pre><div class="code-highlighted"></div></pre></div></div>`, 10);
const CodeInput = props => {
  const merged = solidJs.mergeProps({
    autoHeight: true
  }, props);
  const language = () => merged.language || 'typescript';
  const value = () => merged.value || '';
  let preElement;
  let textAreaElement;
  let wrapperElement;
  let outerElement;

  // Used to detect when the user manually resizes the wrapper with handle
  let wrapperHeight;
  let wrapperWidth;
  const [manualResize, setManualResize] = solidJs.createSignal(false);
  solidJs.onMount(async () => {
    watchResize();
    if (props.autoHeight) {
      autoHeight();
    }
    setBackgroundWrapper();
  });
  function setBackgroundWrapper() {
    const preBackground = window.getComputedStyle(preElement).backgroundColor;
    outerElement.style.backgroundColor = preBackground;
  }
  function setSizes() {
    const {
      height,
      width
    } = getTextareaSize();
    preElement.style.width = `${width}px`;
    preElement.style.height = `${height}px`;
    wrapperElement.style.width = `${width}px`;
    wrapperElement.style.height = `${height}px`;

    // calculate what 1rem is in pixels
    const rem = parseFloat(window.getComputedStyle(document.documentElement).fontSize);
    outerElement.style.width = `${width + rem}px`;
    outerElement.style.height = `${height + rem}px`;
  }
  function getTextareaSize() {
    const {
      height,
      width
    } = textAreaElement.getBoundingClientRect();
    return {
      height,
      width
    };
  }
  function autoHeight() {
    if (manualResize() === true) {
      return;
    }
    wrapperElement.style.height = `0px`;
    wrapperElement.style.height = textAreaElement.scrollHeight + 'px';
  }
  function watchResize() {
    new ResizeObserver(setSizes).observe(textAreaElement);
  }
  function syncScroll() {
    preElement.scrollTop = textAreaElement.scrollTop;
    preElement.scrollLeft = textAreaElement.scrollLeft;
    // Prevents a scrolling issue when the user manually resizes the wrapper
    if (textAreaElement.scrollTop > preElement.scrollTop) {
      textAreaElement.scrollTop = preElement.scrollTop;
    }
    if (textAreaElement.scrollLeft > preElement.scrollLeft) {
      textAreaElement.scrollLeft = preElement.scrollLeft;
    }
  }
  const codeTokens = () => {
    try {
      if (merged.prismJS) {
        if (merged.prismJS.languages[language()]) {
          if (merged.autoHeight) {
            autoHeight();
          }
          const tokens = merged.prismJS.highlight(value(), merged.prismJS.languages[language()], language());
          return tokens;
        } else {
          if (merged.autoHeight) {
            autoHeight();
          }
          return merged.prismJS.util.encode(value()).toString();
        }
      } else if (merged.highlightjs) {
        const tokens = merged.highlightjs.highlight(value(), {
          language: language()
        }).value;
        if (merged.autoHeight) {
          autoHeight();
        }
        return tokens;
      }
    } catch (e) {
      console.error(e);
    }
  };
  function handleKeyDown(event) {
    let input_element = textAreaElement;
    let code = input_element.value;
    if (event.key === 'Tab') {
      handleTabKey(event, input_element, code);
      merged.onChange(input_element.value);
    }
    if (event.key === 'Enter') {
      handleEnterKey(event, input_element);
      merged.onChange(input_element.value);
    }
  }
  async function handleInput(e) {
    merged.onChange(e.target.value);
  }
  function handleMouseDown() {
    const {
      height,
      width
    } = getTextareaSize();
    wrapperHeight = height;
    wrapperWidth = width;
  }
  function handleMouseUp() {
    const {
      height,
      width
    } = getTextareaSize();
    if (height !== wrapperHeight || width !== wrapperWidth) {
      setManualResize(true);
    }
  }
  return (() => {
    const _el$ = _tmpl$.cloneNode(true),
      _el$2 = _el$.firstChild,
      _el$3 = _el$2.firstChild,
      _el$4 = _el$3.nextSibling,
      _el$5 = _el$4.firstChild;
    const _ref$ = outerElement;
    typeof _ref$ === "function" ? web.use(_ref$, _el$) : outerElement = _el$;
    const _ref$2 = wrapperElement;
    typeof _ref$2 === "function" ? web.use(_ref$2, _el$2) : wrapperElement = _el$2;
    _el$2.$$mouseup = handleMouseUp;
    _el$2.$$mousedown = handleMouseDown;
    const _ref$3 = textAreaElement;
    typeof _ref$3 === "function" ? web.use(_ref$3, _el$3) : textAreaElement = _el$3;
    _el$3.addEventListener("scroll", syncScroll);
    _el$3.$$input = handleInput;
    _el$3.$$keydown = handleKeyDown;
    web.setAttribute(_el$3, "spellcheck", false);
    const _ref$4 = preElement;
    typeof _ref$4 === "function" ? web.use(_ref$4, _el$4) : preElement = _el$4;
    web.setAttribute(_el$4, "aria-hidden", true);
    web.effect(_p$ => {
      const _v$ = styles.wrap,
        _v$2 = `${merged.resize ? styles[`resize-${merged.resize}`] : ''}`,
        _v$3 = merged.placeholder || 'Type code here...',
        _v$4 = `language-${language()}`,
        _v$5 = codeTokens();
      _v$ !== _p$._v$ && web.className(_el$2, _p$._v$ = _v$);
      _v$2 !== _p$._v$2 && web.className(_el$3, _p$._v$2 = _v$2);
      _v$3 !== _p$._v$3 && web.setAttribute(_el$3, "placeholder", _p$._v$3 = _v$3);
      _v$4 !== _p$._v$4 && web.className(_el$4, _p$._v$4 = _v$4);
      _v$5 !== _p$._v$5 && (_el$5.innerHTML = _p$._v$5 = _v$5);
      return _p$;
    }, {
      _v$: undefined,
      _v$2: undefined,
      _v$3: undefined,
      _v$4: undefined,
      _v$5: undefined
    });
    web.effect(() => _el$3.value = value());
    return _el$;
  })();
};
web.delegateEvents(["mousedown", "mouseup", "keydown", "input"]);

exports.CodeInput = CodeInput;
//# sourceMappingURL=index.js.map
